// MarkersView.cpp : implementation of the CMarkersView class
//

#include "stdafx.h"
#include "Markers.h"

#include "MarkersDoc.h"
#include "MarkersView.h"
#include "CustomEditCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMarkersView

IMPLEMENT_DYNCREATE(CMarkersView, CBCGPEditView)

BEGIN_MESSAGE_MAP(CMarkersView, CBCGPEditView)
	//{{AFX_MSG_MAP(CMarkersView)
	ON_WM_CREATE()
	ON_WM_CONTEXTMENU()
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_CURSORPOS, OnUpdateCursorPos)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CBCGPEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CBCGPEditView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CBCGPEditView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMarkersView construction/destruction

CMarkersView::CMarkersView()
{
	m_pEdit = NULL;
	m_bLeft = FALSE;
}

CMarkersView::~CMarkersView()
{
}

CBCGPEditCtrl* CMarkersView::CreateEdit ()
{
	m_pEdit = new CCustomEditCtrl;
	return m_pEdit;
}

void CMarkersView::OnInitialUpdate()
{
	CBCGPEditView::OnInitialUpdate();

	ASSERT_VALID (m_pEdit);

	// Load XML settings for syntax highlighting
	const CString strXMLFileName = _T("bcgpedit-cpp.xml");
	m_pEdit->LoadXMLSettings(strXMLFileName);

	// Enable IntelliSense support 
	m_pEdit->EnableIntelliSense(FALSE);

	// Enable breakpoints support:
	m_pEdit->EnableBreakpoints(TRUE);

	// Preset Markers
	COLORREF clrDeleted		= RGB (0, 0, 255);
	COLORREF clrBkDeleted	= RGB (192, 192, 192);
	COLORREF clrChanged		= RGB (255, 0, 0);
	COLORREF clrBkChanged	= RGB (192, 192, 192);
	COLORREF clrInserted	= RGB (0, 128, 0);
	COLORREF clrBkInserted	= RGB (192, 192, 192);
	const CString& strPathName = GetDocument()->GetPathName();
	if (strPathName.IsEmpty())
	{
		RestoreDefaultData();

		if (m_bLeft)
		{
			m_pEdit->ToggleMarker (3, g_dwBCGPEdit_BookMark);
			m_pEdit->ToggleMarker (11, g_dwBCGPEdit_BookMark);
			m_pEdit->ToggleMarker (24, g_dwBCGPEdit_BookMark);

			m_pEdit->ToggleLineMarker (6);

			m_pEdit->SetLineColorMarker (6, clrDeleted, clrBkDeleted, 
				TRUE, 7, g_dwColorStaticMarkerType);
			m_pEdit->SetLineColorMarker (22, clrDeleted, clrBkDeleted, 
				TRUE, 0, g_dwColorStaticMarkerType);
			m_pEdit->SetLineColorMarker (23, clrChanged, clrBkChanged, 
				TRUE, 0, g_dwColorStaticMarkerType);
			OnMarkersNextdiff ();
		}
		else
		{
			m_pEdit->ToggleMarker (3, g_dwBCGPEdit_BookMark);
			m_pEdit->ToggleMarker (24, g_dwBCGPEdit_BookMark);

			m_pEdit->SetLineColorMarker (14, clrInserted, clrBkInserted, 
				TRUE, 3, g_dwColorStaticMarkerType);
			m_pEdit->SetLineColorMarker (23, RGB (255, 0, 0), RGB (192, 192, 192), 
				TRUE, 0, g_dwColorStaticMarkerType);
		}
	}
	else
	{
		m_pEdit->DeleteAllMarkers();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMarkersView diagnostics

#ifdef _DEBUG
void CMarkersView::AssertValid() const
{
	CBCGPEditView::AssertValid();
}

void CMarkersView::Dump(CDumpContext& dc) const
{
	CBCGPEditView::Dump(dc);
}

CMarkersDoc* CMarkersView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMarkersDoc)));
	return (CMarkersDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMarkersView message handlers

int CMarkersView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CBCGPEditView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	ResetDefaultFont();	
	return 0;
}

void CMarkersView::ResetDefaultFont()
{
	ASSERT_VALID (GetEditCtrl ());
	
	HFONT hFont = (HFONT)::GetStockObject(DEFAULT_GUI_FONT);
	CDC* pDC = GetDC();

	CFont* pFont = pDC->SelectObject(CFont::FromHandle(hFont));
	pDC->SelectObject(pFont);
	::DeleteObject(hFont);

	LOGFONT lf;
	pFont->GetLogFont (&lf);

	CString strFontName(_T("Courier New"));
	CopyMemory(lf.lfFaceName,(LPCTSTR)strFontName,(strFontName.GetLength() + 1) * sizeof(TCHAR));
	lf.lfWidth = 0;
	lf.lfEscapement = 0;
	lf.lfOrientation = 0;
	lf.lfWeight = FW_NORMAL;
	lf.lfItalic = 0;
	lf.lfUnderline = 0;
	lf.lfStrikeOut = 0;

	m_Font.CreateFontIndirect(&lf);
	GetEditCtrl ()->SetFont (&m_Font);
}

BOOL CMarkersView::RestoreDefaultData()
{
	ASSERT_VALID (GetEditCtrl ());

	// Load CPP example
	BOOL bRet = FALSE;
	LPCTSTR lpszResourceName = MAKEINTRESOURCE ( m_bLeft ? ID_EXAMPLE1_CPP : ID_EXAMPLE2_CPP);
	ASSERT(lpszResourceName != NULL);

	HINSTANCE hInst = AfxFindResourceHandle(lpszResourceName, _T("Examples"));
	HRSRC hRsrc = ::FindResource(hInst, lpszResourceName, _T("Examples"));
	
	if (hRsrc == NULL)
	{
		ASSERT(FALSE);
	}
	else
	{
		HGLOBAL hGlobal = LoadResource(hInst, hRsrc);

		if (hGlobal == NULL)
		{
			ASSERT(FALSE);
		}
		else
		{
			LPSTR lpszText = (LPSTR)LockResource(hGlobal);

			if (lpszText == NULL)
			{
				ASSERT(FALSE);
			}
			else
			{
				USES_CONVERSION;
				GetEditCtrl ()->SetWindowText(A2T(lpszText));
				bRet = TRUE;
			}

			UnlockResource(hGlobal);
			FreeResource(hGlobal);
		}
	}

	return bRet;
}

void CMarkersView::OnMarkersTogglebookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->ToggleMarker (GetEditCtrl ()->GetCurRow (), g_dwBCGPEdit_BookMark);
}

void CMarkersView::OnMarkersNextbookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->GoToNextMarker(g_dwBCGPEdit_BookMark, TRUE);
}

void CMarkersView::OnMarkersPreviousbookmark() 
{
	ASSERT_VALID (GetEditCtrl ());
	GetEditCtrl ()->GoToNextMarker(g_dwBCGPEdit_BookMark,FALSE);
}

void CMarkersView::OnMarkersTogglebreakpoint() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->ToggleBreakpoint();
}

void CMarkersView::OnMarkersRemoveAllBreakpoints() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->RemoveAllBreakpoints();
}
		   
void CMarkersView::OnMarkersToggleStaticmarker()
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->ToggleStaticMarker();
}

void CMarkersView::OnEditFind() 
{
	OnFindReplace(0, 0);
}

void CMarkersView::OnEditFindnext() 
{
	m_dwFindMask |= FR_DOWN;
	OnFindReplace(0, 0);
}

void CMarkersView::OnEditFindprev() 
{
	m_dwFindMask &= ~FR_DOWN;
	OnFindReplace(0, 0);
}

BOOL CMarkersView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	ASSERT_VALID (GetEditCtrl ());
	GetDocument()->SetModifiedFlag(GetEditCtrl ()->IsModified());
	
	return CBCGPEditView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMarkersView::OnMarkersNextdiff() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->GoToNextMarker(g_dwColorStaticMarkerType, TRUE);
	m_pEdit->MoveLineMarker ();
}

void CMarkersView::OnMarkersPrevdiff() 
{
	ASSERT_VALID (m_pEdit);
	m_pEdit->GoToNextMarker(g_dwColorStaticMarkerType, FALSE);
	m_pEdit->MoveLineMarker ();
}

void CMarkersView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	ASSERT_VALID (GetEditCtrl ());

	CPoint pt(point);

	ScreenToClient(&pt);
	if (pt.x < GetEditCtrl ()->m_nLeftMarginWidth)
	{
		pt.x = GetEditCtrl ()->m_nLeftMarginWidth;
		GetEditCtrl ()->SetCaret(GetEditCtrl ()->HitTest(pt));

		CMenu menu;
		if (menu.LoadMenu(IDR_POPUP_SELECTIONMARGIN))
		{
			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
		}
	}
	else
	{
		CMenu menu;
		if (menu.LoadMenu(IDR_CONTEXT_MENU))
		{
			CMenu* pPopup = menu.GetSubMenu(0);
			ASSERT(pPopup != NULL);

			pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());
		}
	}
}

void CMarkersView::OnUpdateCursorPos(CCmdUI *pCmdUI)
{
	ASSERT_VALID (GetEditCtrl ());

	CString str;
	str.Format(_T("Ln %i, Col %i"), GetEditCtrl ()->GetCurRow ()+1, GetEditCtrl ()->GetCurColumn ()+1);
	pCmdUI->SetText (str);

	pCmdUI->Enable (); 
}
